@@ -1,6 +1,6 @@
 /*****************************************************************************
- * Secure Locate v2.6                                                        *
- * May 21, 2001                                                         *
+ * Secure Locate v2.7                                                        *
+ * January 24, 2003                                                          *
  * Programmed by: Kevin Lindsay                                              *
  * Copyright (c) 1999, 2000, 2001 Kevin Lindsay & Netnation Communications   *
  *                          Inc. & James A. Woods <jwoods@adobe.com>         *
@@ -39,6 +39,9 @@
  *                  database.                                                *
  *           Francis Liu <fxl@enstor.com.au>                                 *
  *                * Solaris Patch                                            *
+ *           Wanderlei Antonio Cavassin <cavassin@conectiva.com.br>          *
+ *                * Fixed a segfault with invalid regex                      *
+ *									     *
  *                                                                           *
  * Report any Bugs to: klindsay@mkintraweb.com                               *
  *                                                                           *
@@ -103,7 +106,7 @@
 
 /* GLOBALS */
 
-#define SL_RELEASE "May 21, 2001"
+#define SL_RELEASE "January 24, 2003"
 #define SL_VERSION "Secure Locate " VERSION " - Released " SL_RELEASE "\n"
 #define GRPFILE "/etc/group"
 #define SLOC_ESC -0x80
@@ -188,7 +191,7 @@ void
 	for (i = 0; i < strlen(progname)-1; i+=1)
 	    printf(" ");
 	
-	printf(
+	fprintf(stdout,
 #ifndef __FreeBSD__
 	       "                  [-c] <[-U <path>] [-u]>\n"
 #else
@@ -278,7 +281,7 @@ int
 	char buf[2];
 	
 	fd = open(database,O_RDONLY);
-	
+
 	if (fd == -1)
 	    return errno;
 	
@@ -386,13 +389,11 @@ void
 
 /* Parse Dash */
 
-void
+char *
     parse_dash(char *option)
 {
 	char *ptr;
-	int i=0;
-	char *database;
-	int res=1;
+	char *regexp_opt;
 	
 	for (ptr = option;*ptr != 0 && *ptr != '=' ; ptr++)
 	    *ptr = toupper(*ptr);
@@ -423,16 +424,18 @@ void
 	
 	if (!strcmp(option,"REGEXP")) {
 		REGEXP = 1;
-		regexp = malloc(strlen(ptr)+1);
-		if (!regexp)
-		    report_error(FATAL,QUIET,"%s: parse_dash: 'regexp': malloc: %s\n",progname,strerror(errno));
-
-		*regexp = 0;
-		strcat(regexp,ptr);
-		while (SLOCATE_PATH && (database = SLOCATE_PATH[i++])) res |= decode_db(database, ptr);
-		exit(!res);
+		regexp_opt = malloc(strlen(ptr)+1);
+		if (!regexp_opt)
+		    report_error(FATAL,QUIET,"%s: parse_dash: 'regexp_opt': malloc: %s\n",progname,strerror(errno));
+		*regexp_opt = 0;
+		strcat(regexp_opt,ptr);
+		regexp = strdup(regexp_opt);
+		// Return regular expression argument here so that the main function can parse
+		// run decode_db properly.
+		return regexp_opt;
 	}
 	
+	return NULL;
 }
 
 
@@ -1078,7 +1081,6 @@ int
 	char *dir = NULL;
 	char *path = NULL;
 	int res;
-	int len = 0;
 	char *str_ptr;
 
 	if (access(codedpath, R_OK) != 0) {
@@ -1092,7 +1094,7 @@ int
 	res = 1;
 	str_ptr = codedpath;
 	
-	while (dir = strtok(str_ptr, "/")) {
+	while ((dir = strtok(str_ptr, "/"))) {
 		strcat(path,"/");
 		strcat(path,dir);
 		if (access(path, R_OK) != 0) {
@@ -1132,7 +1134,7 @@ int
 //	char *chk1;
 //	char tmpch;
 	regex_t *preg=NULL;
-	char *errbuf=NULL;
+	char errbuf[1024];
 	int nmatch=32;
 	regmatch_t pmatch[32];
 	int cur_queries = 0;
@@ -1142,7 +1144,7 @@ int
 	int ptr_offset;
 	char one_char[1];
 	char *begin_ptr;
-	int begin_offset=0;
+        int begin_offset=0;
 	int tot_size = MIN_BLK;
 	int cur_size;
 	int code_tot_size = MIN_BLK;
@@ -1228,6 +1230,11 @@ int
 		
 		if (bytes != 0)
 		    bytes = read(fd,bucket_of_holding+cur_size,MIN_BLK-1);
+		
+		if (bytes == -1) {
+			fprintf(stderr,"%s: decode_db(): read: %s\n",progname,strerror(errno));
+			exit(1);
+		}
 
 		cur_size += bytes;
 
@@ -1239,10 +1246,19 @@ int
 		} else if (code_num > 127)
 		    code_num = code_num - 256;
 
+		/* FIXME sometimes pathlen is < 0 but it shouldn't be.
+		 * corrupt database file? 
+		 * This could be from a bug in frcode() or decode_db(). I
+		 * am leaning toward frcode() at the moment */
+
 		code_ptr += code_num;
-
 		pathlen = code_ptr - codedpath;
-
+		
+		if (pathlen < 0) {
+			fprintf(stderr,"%s: decode_db() aborted. Corrupt database?\n",progname);
+			exit(1);
+		}
+		
 		jump = 0;
 		while (!jump) {
 			ch = *begin_ptr;
@@ -1361,7 +1377,7 @@ int
 	int SPECDIR=0;
 	int ROOTDIR=0;   
 	char *spec_dir=NULL;
-	char *database;
+	char *database = NULL;
 	int i=0;
 	int o_OPT_READY = 0;
 	char *regexp_opt = NULL;
@@ -1491,7 +1507,7 @@ int
 			break;
 		 /* Parse --<argument> type arguments */
 		 case '-':
-			parse_dash(optarg);
+			regexp_opt = parse_dash(optarg);
 			break;
 		 /* Limit the amount of search results */
 		 case 'n':
@@ -1532,14 +1548,16 @@ int
 	else if (ROOTDIR)
 	    res = create_db((char *)NULL);
 	/* search the database(s) */
-	else if (argv[optind] || REGEXP) {
+	else if (REGEXP) {
+		/* We do REGEXP stuff separate even if argv[optind] exists.
+		 * Fixes an exploitable segfault. */
+		int i = 0;
+		while (SLOCATE_PATH && (database = SLOCATE_PATH[i++]))
+		    res |= decode_db(database, regexp_opt);
+	} else if (argv[optind]) {
 		int j = optind;
 		int i = 0;
 		
-		if (REGEXP)
-		    while (SLOCATE_PATH && (database = SLOCATE_PATH[i++]))
-			res |= decode_db(database, regexp_opt);
-
 		while (j < args) {
 			/* while ((database = SLOCATE_PATH[i++])) res |= decode_db(database, argv[j++]);
 			 i = 0;
@@ -1549,7 +1567,7 @@ int
 			    res |= decode_db(database, argv[j]);
 			i = 0; j += 1;
 			
-		}		
+		}
 	}
 	/* Print usage */
 	else
