@@ -39,6 +39,7 @@ static char sccsid[] = "@(#)sl_fts.c	8.6 (Berkeley) 8/14/94";
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <dirent.h>
+#include <sys/dirent.h>
 #include <errno.h>
 #include "sl_fts.h"
 #include <stdlib.h>
@@ -644,7 +645,7 @@ fts_build(sp, type)
 	 */
 	cderrno = 0;
 	if (nlinks || type == BREAD) {
-		if (FCHDIR(sp, dirfd(dirp))) {
+		if (FCHDIR(sp, dirp->dd_fd)) {
 			if (nlinks && type == BREAD)
 				cur->fts_errno = errno;
 			cur->fts_flags |= FTS_DONTCHDIR;
